import os,csv,glob,re

root_dir = "CDROOT"
games_dir = "GAMES"
input_dir = os.path.join(root_dir,games_dir)
menu_dir = os.path.join(root_dir,"MenuData")

database_file = "gameinfo.csv"
empty_database_file = "empty_gameinfo.csv"

gamelist = os.listdir(input_dir)

joypad_type = ["full","no"]

csv_title = ["Name","joypad","extra opts"]
for j in ["J1","J0"]:
    for i in ["blue","yellow","play","green","fwd","bwd","fwdbwd"]:
        csv_title.append(j+" "+i)

keycode_dict = dict()
for i in range(1,9):
    keycode_dict[str(i)] = i
for i in range(0,9):
    keycode_dict["F"+str(i+1)] = i+0x50
for i,c in enumerate("QWERTYUIOP"):
    keycode_dict[c] = i+0x10
for i,c in enumerate("ASDFGHJKL"):
    keycode_dict[c] = i+0x20
for i,c in enumerate("<ZXCVBNM,./"):
    keycode_dict[c] = i+0x30

keycode_dict["Q"] = 0x10
keycode_dict["HELP"] = 0x5F
keycode_dict["RETURN"] = 0x44
keycode_dict["ESC"] = 0x45
keycode_dict["LALT"] = 0x64
keycode_dict["LAMIGA"] = 0x66
keycode_dict["RALT"] = 0x64
keycode_dict["ALT"] = 0x64
keycode_dict["RAMIGA"] = 0x67
keycode_dict["SPACE"] = 0x40
keycode_dict["LSHIFT"] = 0x60
keycode_dict["RSHIFT"] = 0x61
keycode_dict["CTRL"] = 0x63
kclc=dict()
for k,v in keycode_dict.iteritems():
    kclc[k.lower()] = v
keycode_dict.update(kclc)

class GameInfo:
    def __init__(self):
        self.joypad = joypad_type[0]
        self.extra_opts = ""
        self.j1_blue = ""
        self.j1_yellow = ""
        self.j1_play = ""
        self.j1_green = ""
        self.j1_fwd = ""
        self.j1_bwd = ""
        self.j1_fwdbwd = ""
        self.j0_blue = ""
        self.j0_yellow =""
        self.j0_play = ""
        self.j0_green = ""
        self.j0_fwd = ""
        self.j0_bwd = ""
        self.j0_fwdbwd = ""

global nb_warns
nb_warns=0

def get_be_short(s,offset):
    return ord(s[offset])*256+ord(s[offset+1])

def get_c_string(contents,offset):
    i = offset
    while True:
        if ord(contents[i])==0:
            break
        i+=1
    return contents[offset:i].replace("\xff","\n")

def encode_keycode(v,k):
    rval=""
    if v!="":
        if v[0]=='$':
            pass
        else:
            if v in keycode_dict:
                v = "$"+hex(keycode_dict[v])[2:]
            else:
                warn("Unknown keycode %s for %s" % (v,k))
                v="$00"
        rval=k+"="+v+" "
    return rval

def warn(msg):
    global nb_warns
    print("warning: "+msg)
    nb_warns+=1

gamebase_dict = dict()
if os.path.exists(database_file):
    f = open(database_file,"rb")
    cr = csv.reader(f,delimiter=';',quotechar='"')
    f.readline()
    for r in cr:
        g = GameInfo()
        i=0
        name = r[i]; i+=1
        g.joypad = r[i]; i+=1
        g.extra_opts = r[i]; i+=1
        g.j1_blue = r[i]; i+=1
        g.j1_yellow = r[i]; i+=1
        g.j1_play = r[i]; i+=1
        g.j1_green = r[i]; i+=1
        g.j1_fwd = r[i]; i+=1
        g.j1_bwd = r[i]; i+=1
        g.j1_fwdbwd = r[i]; i+=1
        g.j0_blue = r[i]; i+=1
        g.j0_yellow = r[i]; i+=1
        g.j0_play = r[i]; i+=1
        g.j0_green = r[i]; i+=1
        g.j0_fwd = r[i]; i+=1
        g.j0_bwd = r[i]; i+=1
        g.j0_fwdbwd = r[i]; i+=1
        gamebase_dict[name] = g

    f.close()

empty_gamebase_dict = dict()

for gamedir in gamelist:
    slave = glob.glob(os.path.join(input_dir,gamedir,"*.slave"))
    if len(slave)>1:
        warn("Cannot process %s, %d slaves found" % (gamedir,len(slave)))
    elif len(slave)==1:
        data_option = ""
        datapat = glob.glob(os.path.join(input_dir,gamedir,"data*"))
        datadir=[]
        for i in datapat:
            if os.path.isdir(i):
                datadir.append(i)
        if len(datadir)>1:
            warn("Cannot process %s, %d data dirs found" % (gamedir,len(datadir)))
        else:
            if len(datadir)==1:
                data_option = "DATA="+os.path.basename(datadir[0])

            slave = slave[0]
            f = open(slave,"rb")
            header = f.read(0x20)
            contents = f.read()
            f.close()
            h = contents[4:12]
            if h!="WHDLOADS":
                warn("%s: not a slave" % slave)
            slave_version = get_be_short(contents,12)
            slave_info = ""
            slave_copyright = ""
            if slave_version>9:
                slave_name_offset = get_be_short(contents,0x24)
                # remove suspiciously long names or ISO image will be corrupt and crash
                gfnu = get_c_string(contents,slave_name_offset).replace("-->","").replace("<--","")
                game_full_name = re.sub(r"(........)[:\-/].*",r"\1",gfnu).strip()
                slave_copy_offset = get_be_short(contents,0x26)
                slave_copyright = get_c_string(contents,slave_copy_offset)
                slave_info_offset = get_be_short(contents,0x28)
                slave_info = get_c_string(contents,slave_info_offset)
                # wrap to 40 chars
                silines = slave_info.split("\n")
                wrap_col = 40
                i = 0
                while i < len(silines):
                    l = silines[i]
                    if len(l)<wrap_col:
                        pass
                    else:
                        last_space = l[:wrap_col].rfind(" ")
                        if last_space==-1:
                            limit = wrap_col
                        else:
                            limit = last_space

                        extra_line = l[limit:].strip()
                        silines[i] = l[:limit]
                        silines.insert(i+1,extra_line)
                    i+=1
                slave_info = "\n".join(silines)
            else:
                game_full_name = os.path.splitext(os.path.basename(slave))[0]
            mdprefix = os.path.join(menu_dir,game_full_name)
            start_file = mdprefix+".start"
            txt_file = mdprefix+".txt"
            f=open(txt_file,"wb")
            f.write(slave_copyright+"\n"+slave_info)
            f.close()
            f=open(start_file,"wb")
            slave_on_cd = "CD0:"+games_dir+"/"+gamedir+"/"+os.path.basename(slave)
            line = 'cd32load "'+slave_on_cd+'" '
            if game_full_name in gamebase_dict:
                g = gamebase_dict[game_full_name]
                line += g.extra_opts+" "
                if g.joypad=="no":
                    line += "NOJOYPAD "
                line+=encode_keycode(g.j0_blue,"JOY0BLUE")
                line+=encode_keycode(g.j0_green,"JOY0GREEN")
                line+=encode_keycode(g.j0_yellow,"JOY0YELLOW")
                line+=encode_keycode(g.j0_play,"JOY0PLAY")
                line+=encode_keycode(g.j0_fwd,"JOY0FWD")
                line+=encode_keycode(g.j0_bwd,"JOY0BWD")
                line+=encode_keycode(g.j0_fwdbwd,"JOY0FWDBWD")
                line+=encode_keycode(g.j1_blue,"JOY1BLUE")
                line+=encode_keycode(g.j1_green,"JOY1GREEN")
                line+=encode_keycode(g.j1_yellow,"JOY1YELLOW")
                line+=encode_keycode(g.j1_play,"JOY1PLAY")
                line+=encode_keycode(g.j1_fwd,"JOY1FWD")
                line+=encode_keycode(g.j1_bwd,"JOY1BWD")
                line+=encode_keycode(g.j1_fwdbwd,"JOY1FWDBWD")
            else:
                empty_gamebase_dict[game_full_name] = GameInfo()
            line += data_option
            f.write(line.strip()+"\n")
            f.close()

if len(empty_gamebase_dict)>0:
    f = open(empty_database_file,"wb")
    cw = csv.writer(f,delimiter=';',quotechar='"')
    cw.writerow(csv_title)
    for n in sorted(empty_gamebase_dict.keys()):
        g = empty_gamebase_dict[n]
        cw.writerow([n,"full","",g.j1_blue,g.j1_yellow,g.j1_play,g.j1_green,g.j1_fwd,g.j1_bwd,g.j1_fwdbwd,
        g.j0_blue,g.j0_yellow,g.j0_play,g.j0_green,g.j0_fwd,g.j0_bwd,g.j0_fwdbwd])

    f.close()

if nb_warns>0:
    print("%d warning(s) found" % nb_warns)